@@ -151,18 +151,18 @@ class Hub(greenlet):
     def switch(self):
         cur = getcurrent()
         assert cur is not self, 'Cannot switch to MAINLOOP from MAINLOOP'
-        exc_info = sys.exc_info()
+        exc_type, exc_value = sys.exc_info()[:2]
         try:
-            sys.exc_clear()
             switch_out = getattr(cur, 'switch_out', None)
             if switch_out is not None:
                 try:
                     switch_out()
                 except:
                     traceback.print_exc()
+            sys.exc_clear()
             return greenlet.switch(self)
         finally:
-            core.set_exc_info(*exc_info)
+            core.set_exc_info(exc_type, exc_value)
 
     def run(self):
         global _threadlocal
@@ -170,7 +170,7 @@ class Hub(greenlet):
         try:
             self.keyboard_interrupt_signal = signal(2, core.active_event, MAIN.throw, KeyboardInterrupt)
         except IOError:
-            pass  # no signal() on windows
+            pass  # no signal() on Windows
         try:
             loop_count = 0
             while True:
